#python TFIDF.py <Test file>

import logging
import sys
from gensim import corpora, models, similarities
from pprint import pprint
from collections import defaultdict
from sklearn.ensemble import RandomForestClassifier
from sklearn.multiclass import OneVsRestClassifier
from sklearn.linear_model import LogisticRegression
from sklearn.cross_validation import train_test_split
from sklearn.ensemble import GradientBoostingClassifier
from sklearn import metrics
from sklearn.svm import SVC
import numpy as np

logging.basicConfig(format='%(asctime)s : %(levelname)s : %(message)s', level=logging.INFO)

def get_label_text(bow_str):

    text = bow_to_text[bow_str]
    label = text_to_label[text]
    return label

def get_prot_label_result(X,classifier,filename):
   
    fout = open(filename,"w")
    prot_predicted_label = {}
    predicted = classifier.predict(X)
    
    #probs = classifier.predict_proba(X)
    for i in range(len(X)):
        x = X[i]
        x_str = " ".join([str(j) for j in x])
        bow_str = org_feature_to_bow[x_str]
        text = org_bow_to_text[bow_str]
        prots = org_text_to_prot[text]
        text_label = predicted[i]
        for prot in prots:
            if not prot_predicted_label.has_key(prot):
                prot_predicted_label[prot] = []
                prot_predicted_label[prot].append(text_label)
            else:
                prot_predicted_label[prot].append(text_label) 

    for prot in prot_predicted_label:
        MP_count = sum(prot_predicted_label[prot])
#        if MP_count >= len(prot_predicted_label[prot])-MP_count:	#for 50% majority vote	
        print MP_count, len(prot_predicted_label[prot])
        if MP_count > len(prot_predicted_label[prot]) * 0.9:
            fout.write(prot+" 1\n")
        else:
            fout.write(prot+" 0\n")
 
organism_data = open(sys.argv[1],"r")
org_text_to_prot = {}
org_texts = []
while 1:
    line = organism_data.readline()
    if not line:
        break
    (id,text) = line.strip().split("\t")
    if not org_text_to_prot.has_key(text):
        org_text_to_prot[text] = set()
        org_text_to_prot[text].add(id)
        org_texts.append(text.split())
    else:
        org_text_to_prot[text].add(id)     

MP_file = open("MP_desc.dat","r")
text_to_prot = {}
text_to_label = {}
texts = []
pos_texts = []
neg_texts = []
while 1:
    line = MP_file.readline()
    if not line:
        break
    (id,text) = line.strip().split("\t")
    if not text_to_prot.has_key(text):
        text_to_prot[text] = set()
        text_to_prot[text].add(id)
        text_to_label[text]=1
        texts.append(text.split())
        pos_texts.append(text.split())
    else:
        text_to_prot[text].add(id)

    
NonMP_file = open("NMP_desc.dat","r")
while 1:
    line = NonMP_file.readline()
    if not line:
        break
    (id,text) = line.strip().split("\t")
    if not text_to_prot.has_key(text):
        text_to_prot[text] = set()
        text_to_prot[text].add(id)
        text_to_label[text]=0
        texts.append(text.split())
        neg_texts.append(text.split())
    else:
        text_to_prot[text].add(id)

print ("Done reading training data")

dictionary = corpora.Dictionary(texts)
pos_dictionary = corpora.Dictionary(pos_texts)
neg_dictionary = corpora.Dictionary(neg_texts)
bow_to_text = {}

corpus = [] 
for text in texts:
    bow = dictionary.doc2bow(text)
    store_text = " ".join(text)
    if not bow_to_text.has_key(store_text):
        bow_to_text[str(bow)] = store_text
    corpus.append(bow)

pos = set(pos_dictionary.values())
neg = set(neg_dictionary.values())
pos = pos-neg
org_dictionary = corpora.Dictionary(org_texts)

org_bow_to_text = {}

org_corpus = [] 
for text in org_texts:
    bow = dictionary.doc2bow(text)
    store_text = " ".join(text)
    if not org_bow_to_text.has_key(store_text):
        org_bow_to_text[str(bow)] = store_text
    org_corpus.append(bow)


tfidf = models.TfidfModel(corpus)

print ("Done Populating TFIDF Model")

X = []
y = []
feature_to_bow = {}
non_zero_index = set()
for bow in corpus:
    doc = tfidf[bow]
    bow_str = str(bow)
    hash_ = dict(doc)
    label = get_label_text(bow_str)

    y.append(label)
    x = []
    for word,id in dictionary.token2id.iteritems():
        if id in hash_:
            x.append(hash_[id])
        else:
            x.append(0)
    x_str = " ".join([str(i) for i in x])
    if label==1:
        non_zero = [i for i, j in enumerate(x) if j != 0]
        str1 = " ".join([str(i) for i in non_zero])
        non_zero_index.add(str1)
    feature_to_bow[x_str]=bow_str
    X.append(x)



Xtest = []
org_feature_to_bow = {}
non_zero_index_org = set()
for bow in org_corpus:
    doc = tfidf[bow]
    bow_str = str(bow)
    hash_ = dict(doc)
    x = []
    for word,id in dictionary.token2id.iteritems():
        if id in hash_:
            x.append(hash_[id])
        else:
            x.append(0)
    #if np.sum(x) > 0:
    #    print np.sum(x)
    x_str = " ".join([str(i) for i in x])
    non_zero = [i for i, j in enumerate(x) if j != 0]
    str1 = " ".join([str(i) for i in non_zero])
    non_zero_index_org.add(str1)
    org_feature_to_bow[x_str]=bow_str
    Xtest.append(x)

print ("Done generating tfidf-based feature vectors")
classifier = LogisticRegression(C = 1, solver = 'liblinear', max_iter = 100)
classifier.fit(X, y)
get_prot_label_result(Xtest,classifier,'TFIDF_logistic_'+sys.argv[1])

classifier = RandomForestClassifier(n_estimators = 500)
classifier.fit(X, y)
get_prot_label_result(Xtest,classifier,'TFIDF_randomforest_'+sys.argv[1])  

classifier = GradientBoostingClassifier(learning_rate=0.1, n_estimators = 100)
classifier.fit(X, y)
get_prot_label_result(Xtest,classifier,'TFIDF_gradientboost_'+sys.argv[1]) 


classifier = SVC(kernel = 'linear', C = 2)			#abstract/function
classifier.fit(X, y)
get_prot_label_result(Xtest,classifier,'TFIDF_SVM_'+sys.argv[1]) 

print ("Done predictiong for new data")

