# USAGE:
# python TFIDF.py <MP File> <Non_MP FILE>

#example:

# python TFIDF.py ../../Data/Cleaned/Title/title_MP.dat.processed ../../Data/Cleaned/Title/title_NMP.dat.processed


import sys
from gensim import corpora, models, similarities

from collections import defaultdict
from sklearn.ensemble import RandomForestClassifier
from sklearn.multiclass import OneVsRestClassifier
from sklearn.linear_model import LogisticRegression
from sklearn.cross_validation import train_test_split
from sklearn.ensemble import GradientBoostingClassifier
from sklearn import metrics
from sklearn.svm import SVC

import numpy as np
import time


MP_file = open(sys.argv[1],"r")

protein_id_to_cls = {}
texts = []
model_time = 0.0
infer_time = 0.0
CVtime = []
index = 1
while 1:
    line = MP_file.readline()
    if not line:
        break
    (id,text) = line.strip().split("\t")
    protein_id_to_cls[index]=1
    index+=1
    texts.append(text.split())
    
NonMP_file = open(sys.argv[2],"r")
while 1:
    line = NonMP_file.readline()
    if not line:
        break
    print line
    (id,text) = line.strip().split("\t")
    
    protein_id_to_cls[index]=0
    index+=1
    texts.append(text.split())

#print texts
start = time.time()
dictionary = corpora.Dictionary(texts)
corpus = [] 
for text in texts:
    corpus.append(dictionary.doc2bow(text))

tfidf = models.TfidfModel(corpus)
corpus_tfidf = tfidf[corpus]
end = time.time()
model_time = (float(end - start))
index=1
X = []
y = []
unique = {}
start = time.time()
for doc in corpus_tfidf:
   
    hash_ = dict(doc)
    y.append(protein_id_to_cls[index])
    x = []
    for word,id in dictionary.token2id.iteritems():
        
        if id in hash_:
           
            x.append(hash_[id])
        else:
           
            x.append(0)
    X.append(x)
    index+=1
   
end = time.time()
infer_time = (float(end - start))

print model_time, infer_time
exit()
print np.asarray(X).shape
print np.sum(y)

d = zip(x,y)
new_array = [tuple(row) for row in d]
uniques = np.unique(new_array)

x,y = zip(*(d))

print np.asarray(X).shape
print np.sum(y)

f1LR = []
preLR = []
recalLR = []

f1RF = []
preRF = []
recalRF = []

f1SVM = []
preSVM = []
recalSVM = []

f1GBM = []
preGBM = []
recalGBM = []

for c in range(3):
    print "fold = ", c

    
    X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=0.2, random_state=0+c*10)

    #if test data dont have 0 and 1 label, skip
    if np.sum(y_test) == len(y_test):	#only 1 class
        continue
    if np.sum(y_test) == 0:		#only 0 class
        continue

    T = len(y_test)
    P = sum(y_test)
    N = T - P

    start = time.time()
    classifier = LogisticRegression(C = 1, solver = 'liblinear', max_iter = 100)
    classifier.fit(X_train, y_train)
    predicted = classifier.predict(X_test)
    probs = classifier.predict_proba(X_test)
    #R =  metrics.classification_report(y_test, predicted)

    preLR.append(((metrics.precision_score(y_test, predicted, pos_label=1)*P) + (metrics.precision_score(y_test, predicted, pos_label=0)*N))/T)
    recalLR.append(((metrics.recall_score(y_test, predicted, pos_label=1)*P) + (metrics.recall_score(y_test, predicted, pos_label=0)*N))/T)
    f1LR.append(((metrics.f1_score(y_test, predicted, pos_label=1)*P) + (metrics.f1_score(y_test, predicted, pos_label=0)*N))/T)
    print "done LR"

    cls = RandomForestClassifier(n_estimators = 500)
    cls.fit(X_train, y_train)
    predicted = cls.predict(X_test)

    preRF.append(((metrics.precision_score(y_test, predicted, pos_label=1)*P) + (metrics.precision_score(y_test, predicted, pos_label=0)*N))/T)
    recalRF.append(((metrics.recall_score(y_test, predicted, pos_label=1)*P) + (metrics.recall_score(y_test, predicted, pos_label=0)*N))/T)
    f1RF.append(((metrics.f1_score(y_test, predicted, pos_label=1)*P) + (metrics.f1_score(y_test, predicted, pos_label=0)*N))/T)
    print "done RF"
    
    # corss validated classifier
    if "title" in sys.argv[1]:
        cls = SVC(kernel = 'rbf', C = 6, gamma = 0.5)	#title
    else:
        cls = SVC(kernel = 'linear', C = 2)			#abstract/function
        print "here"

    print cls

    cls.fit(X_train, y_train)
    predicted = cls.predict(X_test)

    
    preSVM.append(((metrics.precision_score(y_test, predicted, pos_label=1)*P) + (metrics.precision_score(y_test, predicted, pos_label=0)*N))/T)
    recalSVM.append(((metrics.recall_score(y_test, predicted, pos_label=1)*P) + (metrics.recall_score(y_test, predicted, pos_label=0)*N))/T)
    f1SVM.append(((metrics.f1_score(y_test, predicted, pos_label=1)*P) + (metrics.f1_score(y_test, predicted, pos_label=0)*N))/T)
    print "done SVM"

    cls = GradientBoostingClassifier(learning_rate=0.1, n_estimators = 100)
    cls.fit(X_train, y_train)
    predicted = cls.predict(X_test)

    preGBM.append(((metrics.precision_score(y_test, predicted, pos_label=1)*P) + (metrics.precision_score(y_test, predicted, pos_label=0)*N))/T)
    recalGBM.append(((metrics.recall_score(y_test, predicted, pos_label=1)*P) + (metrics.recall_score(y_test, predicted, pos_label=0)*N))/T)
    f1GBM.append(((metrics.f1_score(y_test, predicted, pos_label=1)*P) + (metrics.f1_score(y_test, predicted, pos_label=0)*N))/T)
    print "done GBM"
    end = time.time()
    CVtime.append(float(end - start))
fout = open('Result_TFIDF_abstract.txt', 'w')

fout.write("5 fold LogisticRegression:");
fout.write("F1 = %f\n" % np.mean(f1LR))
fout.write("precision = %f\n" % np.mean(preLR))
fout.write("recall = %f\n" % np.mean(recalLR))

fout.write("\n5 fold Random Forest:");
fout.write("F1 = %f\n" % np.mean(f1RF))
fout.write("precision = %f\n" % np.mean(preRF))
fout.write("recall = %f\n" % np.mean(recalRF))

fout.write("\n5 fold SVM:");
fout.write("F1 = %f\n" % np.mean(f1SVM))
fout.write("precision = %f\n" % np.mean(preSVM))
fout.write("recall = %f\n" % np.mean(recalSVM))

fout.write("\n5 fold GBM:");
fout.write("F1 = %f\n" % np.mean(f1GBM))
fout.write("precision = %f\n" % np.mean(preGBM))
fout.write("recall = %f\n" % np.mean(recalGBM))

time =  model_time +infer_time+ np.mean(CVtime)
fout.write("Model time = "+ str(model_time) +" Sec, Infer time = "+ str(infer_time) +" Sec, Classification time =" + str(np.mean(CVtime))+ "Sec\n")
fout.write("Total time = %f Sec\n" % time)
fout.close();


