# USAGE:
# python LDA_protein_level_majority_vote_RF.py <Classlabel_File> <MP File> <Non_MP FILE>

import logging
import sys
from gensim import corpora, models, similarities
from pprint import pprint
from collections import defaultdict
from sklearn.ensemble import RandomForestClassifier
from sklearn.multiclass import OneVsRestClassifier
from sklearn.linear_model import LogisticRegression
from sklearn.cross_validation import train_test_split
from sklearn.ensemble import GradientBoostingClassifier
from sklearn import metrics
from sklearn.svm import SVC
import numpy as np

logging.basicConfig(format='%(asctime)s : %(levelname)s : %(message)s', level=logging.INFO)

def get_label_text(bow_str):

    text = bow_to_text[bow_str]
    label = text_to_label[text]
    return label

def get_prot_label_result(X,y,classifier,prot_test):
   
    prot_predicted_label = {}
    predicted = classifier.predict(X)
    
    #probs = classifier.predict_proba(X)
    for i in range(len(y)):
        x = X[i]
        x_str = " ".join([str(j) for j in x])
        bow_str = feature_to_bow[x_str]
        text = bow_to_text[bow_str]
        prots = text_to_prot[text]
        text_label = predicted[i]
        for prot in prots:
            if not prot_test.has_key(prot):
                continue
            if not prot_predicted_label.has_key(prot):
                prot_predicted_label[prot] = []
                prot_predicted_label[prot].append(text_label)
            else:
                prot_predicted_label[prot].append(text_label) 
    prot_actual = []
    prot_predicted = []
    
    #print len(prot_to_label)
    #print len(prot_predicted_label)
    #print len(prot_test)
    #exit()
    for prot in prot_predicted_label:
        prot_actual.append(prot_to_label[prot])
        MP_count = sum(prot_predicted_label[prot])
#        if MP_count >= len(prot_predicted_label[prot])-MP_count:	#for 50% majority vote	
        if MP_count > len(prot_predicted_label[prot]) * 0.8:
            prot_predicted.append(1)
        else:
            prot_predicted.append(0)            

    T = len(prot_actual)
    P = sum(prot_actual)
    N = T - P
    print metrics.classification_report(prot_actual, prot_predicted)
    precision = ((metrics.precision_score(prot_actual, prot_predicted, pos_label=1)*P) + (metrics.precision_score(prot_actual, prot_predicted, pos_label=0)*N))/T
    recall = ((metrics.recall_score(prot_actual, prot_predicted, pos_label=1)*P) + (metrics.recall_score(prot_actual, prot_predicted, pos_label=0)*N))/T
    fscore = ((metrics.f1_score(prot_actual, prot_predicted, pos_label=1)*P) + (metrics.f1_score(prot_actual, prot_predicted, pos_label=0)*N))/T
    return precision,recall,fscore

All_prot_label_file = open(sys.argv[1],"r")
prot_to_label = {}

while 1:
    line = All_prot_label_file.readline()
    if not line:
        break
    p,l = line.strip().split()
    prot_to_label[p]=int(l)
    
         

MP_file = open(sys.argv[2],"r")
text_to_prot = {}
text_to_label = {}
texts = []
while 1:
    line = MP_file.readline()
    if not line:
        break
    (id,text) = line.strip().split("\t")
    if not text_to_prot.has_key(text):
        text_to_prot[text] = set()
        text_to_prot[text].add(id)
        text_to_label[text]=1
        texts.append(text.split())
    else:
        text_to_prot[text].add(id)

    
NonMP_file = open(sys.argv[3],"r")
while 1:
    line = NonMP_file.readline()
    if not line:
        break
    (id,text) = line.strip().split("\t")
    if not text_to_prot.has_key(text):
        text_to_prot[text] = set()
        text_to_prot[text].add(id)
        text_to_label[text]=0
        texts.append(text.split())
    else:
        text_to_prot[text].add(id)

dictionary = corpora.Dictionary(texts)
bow_to_text = {}

corpus = [] 
for text in texts:
    bow = dictionary.doc2bow(text)
    store_text = " ".join(text)
    if not bow_to_text.has_key(store_text):
        bow_to_text[str(bow)] = store_text
    corpus.append(bow)


tfidf = models.TfidfModel(corpus)
corpus_tfidf = tfidf[corpus]

if "title" in sys.argv[1]:
    num_topics = 150     # title
elif "abstract" in sys.argv[1]:
    num_topics = 130	#abstract
else:
    num_topics = 150     #function

topic = models.LdaModel(corpus_tfidf,id2word=dictionary,num_topics=num_topics,iterations=5000)

X = []
y = []
feature_to_bow = {}
for bow in corpus:
    doc = tfidf[bow]
    bow_str = str(bow)
    topic_dist = dict(topic.get_document_topics(doc,minimum_probability=0.000))
    label = get_label_text(bow_str)
    #print bow_to_text[bow_str],get_label_text(bow_str)
    y.append(label)
    x = []
    for id in range(0,num_topics):
        #print word,id
        if topic_dist.has_key(id):
            x.append(topic_dist[id])
        else:
            #output.write("0 ")
            x.append(0)
    x_str = " ".join([str(i) for i in x])
    feature_to_bow[x_str]=bow_str
    X.append(x)

#exit()
print np.asarray(X).shape
print np.sum(y)

f1LR = []
preLR = []
recalLR = []

f1RF = []
preRF = []
recalRF = []

f1SVM = []
preSVM = []
recalSVM = []

f1GBM = []
preGBM = []
recalGBM = []

for c in range(5):
    print "fold = ", c
    prot_train = {}
    prot_test = {}
    prot_index = np.asarray(range(0,len(prot_to_label),1))
    np.random.seed(0+c*10)
    np.random.shuffle(prot_index)
    
    test_fold = 20*len(prot_to_label)/100
    for i in range(0,len(prot_to_label)):
        prot = prot_to_label.keys()[prot_index[i]]
        if i < test_fold:
            prot_test[prot]=prot_to_label[prot]
        else:
            prot_train[prot]=prot_to_label[prot]

    X_train = []
    X_test = []
    y_train = []
    y_test = []
    for i in range(len(y)):
        x = X[i]
        x_str = " ".join([str(j) for j in x])
        bow_str = feature_to_bow[x_str]
        text = bow_to_text[bow_str]
        prots = text_to_prot[text]
        if len(prots.intersection(set(prot_test.keys()))) > 0:
            X_test.append(x)
            y_test.append(y[i])
        else:
            X_train.append(x)
            y_train.append(y[i])
    #X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=0.2, random_state=0+c*10)

    
    #if test data dont have 0 and 1 label, skip
    if np.sum(y_test) == len(y_test):	#only 1 class
        continue
    if np.sum(y_test) == 0:		#only 0 class
        continue

    print len(X_test)
    print len(X_train)
    #exit()

    if "title" in sys.argv[1]:
        cls = RandomForestClassifier(n_estimators = 250)	#title
    elif "abstract" in sys.argv[1]:
        cls = RandomForestClassifier(n_estimators = 750)	#abstract

    cls.fit(X_train, y_train)
    predicted = cls.predict(X_test)

    precision,recall,fscore = get_prot_label_result(X_test,y_test,cls,prot_test)    
    preRF.append(precision)
    recalRF.append(recall)
    f1RF.append(fscore)
    print "done RF"
    

fout = open('Result_LDA_RF.txt', 'a')


fout.write("\n5 fold Random Forest:"+sys.argv[1]+" "+sys.argv[4]+"\n");
fout.write("F1 = %f\n" % np.mean(f1RF))
fout.write("precision = %f\n" % np.mean(preRF))
fout.write("recall = %f\n" % np.mean(recalRF))

fout.close();


